"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const apollo_client_1 = require("apollo-client");
const apollo_link_1 = require("apollo-link");
const apollo_link_ws_1 = require("apollo-link-ws");
const apollo_link_error_1 = require("apollo-link-error");
const apollo_link_context_1 = require("apollo-link-context");
const apollo_link_http_1 = require("apollo-link-http");
const apollo_cache_inmemory_1 = require("apollo-cache-inmemory");
const apollo_utilities_1 = require("apollo-utilities");
const subscriptions_transport_ws_1 = require("subscriptions-transport-ws");
const ws_1 = __importDefault(require("ws"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const getErrorMessage = ({ graphQLErrors, networkError, operation }) => `GraphQL Error${networkError && networkError.statusCode
    ? ` (${networkError.statusCode}): `
    : ': '}${networkError && networkError.message.replace(/\.$/, '')}${graphQLErrors && graphQLErrors.length
    ? `:\n${graphQLErrors.map(({ message }) => message).join('\n')}`
    : ''}${operation ? `\n@ ${operation.operationName}` : ''}${operation && operation.variables ? `(${util_1.inspect(operation.variables)})` : ''}`;
exports.createLink = ({ uri, wsUri, wsOptions, mutationLink, queryLink, subscriptionLink, }) => {
    const wsOperations = [];
    if (mutationLink === 'ws') {
        wsOperations.push('mutation');
    }
    if (queryLink === 'ws') {
        wsOperations.push('query');
    }
    if (subscriptionLink === 'ws') {
        wsOperations.push('subscription');
    }
    const httpLink = apollo_link_http_1.createHttpLink({ uri, fetch: node_fetch_1.default });
    const wsClient = new subscriptions_transport_ws_1.SubscriptionClient(wsUri, {
        ...wsOptions,
        reconnect: true,
    }, ws_1.default);
    const wsLink = new apollo_link_ws_1.WebSocketLink(wsClient);
    return wsOperations.length
        ? apollo_link_1.ApolloLink.split(({ query }) => (({ kind, operation }) => kind === 'OperationDefinition' && wsOperations.includes(operation))(apollo_utilities_1.getMainDefinition(query)), wsLink, httpLink)
        : httpLink;
};
exports.createClient = ({ uri, wsUri, wsOptions, mutationLink = 'http', queryLink = 'http', subscriptionLink = 'ws', cache, headers, log, }) => {
    const transportLink = exports.createLink({
        uri,
        wsUri,
        wsOptions,
        mutationLink,
        queryLink,
        subscriptionLink,
    });
    const link = apollo_link_1.ApolloLink.from([
        apollo_link_error_1.onError((error) => (log || console).error(getErrorMessage(error))),
        ...(headers && Object.keys(headers).length
            ? [
                apollo_link_context_1.setContext((_request, context) => ({
                    headers: { ...context.headers, ...headers },
                })),
            ]
            : []),
        transportLink,
    ]);
    if (log)
        log.log(`Connecting to ${uri}${wsUri ? ` with subscriptions at ${wsUri}` : ''}`, ...(headers && Object.keys(headers).length ? [headers] : []));
    return new apollo_client_1.ApolloClient({ cache: cache || new apollo_cache_inmemory_1.InMemoryCache(), link });
};
//# sourceMappingURL=kikstart-apollo-client.js.map